package dev.fyloz.trial.colorrecipesexplorer.model

import com.fasterxml.jackson.annotation.JsonIgnore
import dev.fyloz.trial.colorrecipesexplorer.model.validation.NullOrNotBlank
import java.util.*
import javax.persistence.*
import javax.validation.constraints.NotNull

private const val RECIPE_STEP_ID_NULL_MESSAGE = "Un identifiant est requis"
private const val RECIPE_STEP_RECIPE_NULL_MESSAGE = "Une recette est requise"
private const val RECIPE_STEP_MESSAGE_NULL_MESSAGE = "Un message est requis"

@Entity
data class RecipeStep(
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    override val id: Long?,

    @JsonIgnore
    @ManyToOne
    val recipe: Recipe?,

    val message: String
) : Model {
    constructor(recipe: Recipe?, message: String) : this(null, recipe, message)

    override fun equals(other: Any?): Boolean =
        other is RecipeStep && other.recipe == recipe && other.message == message

    override fun hashCode(): Int = Objects.hash(recipe, message)
}


open class RecipeStepSaveDto(
    @field:NotNull(message = RECIPE_STEP_RECIPE_NULL_MESSAGE)
    val recipe: Recipe,

    @field:NotNull(message = RECIPE_STEP_MESSAGE_NULL_MESSAGE)
    val message: String
) : EntityDto<RecipeStep> {
    override fun toEntity(): RecipeStep = RecipeStep(null, recipe, message)
}


open class RecipeStepUpdateDto(
    @field:NotNull(message = RECIPE_STEP_ID_NULL_MESSAGE)
    val id: Long,

    val recipe: Recipe?,

    @field:NullOrNotBlank(message = RECIPE_STEP_MESSAGE_NULL_MESSAGE)
    val message: String?
) : EntityDto<RecipeStep> {
    override fun toEntity(): RecipeStep = RecipeStep(id, recipe, message ?: "")
}

// ==== DSL ====
fun recipeStep(
    id: Long? = null,
    recipe: Recipe? = TODO(), // TODO change default when recipe DSL is done
    message: String = "message",
    op: RecipeStep.() -> Unit = {}
) = RecipeStep(id, recipe, message).apply(op)

fun recipeStepSaveDto(
    recipe: Recipe = TODO(),   // TODO change default when recipe DSL is done
    message: String = "message",
    op: RecipeStepSaveDto.() -> Unit = {}
) = RecipeStepSaveDto(recipe, message).apply(op)

fun recipeStepUpdateDto(
    id: Long = 0L,
    recipe: Recipe? = TODO(),
    message: String? = "message",
    op: RecipeStepUpdateDto.() -> Unit = {}
) = RecipeStepUpdateDto(id, recipe, message).apply(op)
